#ifndef FIGURETYPE_SOLDIER_H
#define FIGURETYPE_SOLDIER_H

#include "figure/figure.h"

void figure_military_standard_action(figure *f);

void figure_soldier_action(figure *f);

#endif // FIGURETYPE_SOLDIER_H
