#ifndef WIGET_CITY_OVERLAY_EDUCATION_H
#define WIGET_CITY_OVERLAY_EDUCATION_H

#include "city_overlay.h"

const city_overlay *city_overlay_for_education(void);

const city_overlay *city_overlay_for_school(void);

const city_overlay *city_overlay_for_library(void);

const city_overlay *city_overlay_for_academy(void);

#endif // WIGET_CITY_OVERLAY_EDUCATION_H
