#ifndef FIGURETYPE_MAINTENANCE_H
#define FIGURETYPE_MAINTENANCE_H

#include "figure/figure.h"

void figure_engineer_action(figure *f);

void figure_prefect_action(figure *f);

void figure_worker_action(figure *f);

#endif // FIGURETYPE_MAINTENANCE_H
