#ifndef CITY_HEALTH_H
#define CITY_HEALTH_H

int city_health(void);

void city_health_change(int amount);

void city_health_update(void);

void city_health_reset_hospital_workers(void);

void city_health_add_hospital_workers(int amount);

#endif // CITY_HEALTH_H
