#ifndef WINDOW_BUILDING_TERRAIN_H
#define WINDOW_BUILDING_TERRAIN_H

#include "common.h"

void window_building_draw_no_people(building_info_context *c);

void window_building_draw_terrain(building_info_context *c);

#endif // WINDOW_BUILDING_TERRAIN_H
