#ifndef WIGET_CITY_OVERLAY_OTHER_H
#define WIGET_CITY_OVERLAY_OTHER_H

#include "city_overlay.h"

const city_overlay *city_overlay_for_religion(void);

const city_overlay *city_overlay_for_tax_income(void);

const city_overlay *city_overlay_for_food_stocks(void);

const city_overlay *city_overlay_for_water(void);

const city_overlay *city_overlay_for_desirability(void);

#endif // WIGET_CITY_OVERLAY_OTHER_H
