#ifndef GRAPHICS_BUTTON_H
#define GRAPHICS_BUTTON_H

void button_none(int param1, int param2);

void button_border_draw(int x, int y, int width_pixels, int height_pixels, int has_focus);

#endif // GRAPHICS_BUTTON_H
