#ifndef CITY_SOUND_H
#define CITY_SOUND_H

int city_sound_update_march_enemy(void);

int city_sound_update_march_horse(void);

int city_sound_update_march_wolf(void);

int city_sound_update_shoot_arrow(void);

int city_sound_update_hit_wolf(void);

int city_sound_update_hit_soldier(void);

int city_sound_update_hit_axe(void);

int city_sound_update_hit_club(void);

int city_sound_update_hit_spear(void);

int city_sound_update_hit_elephant(void);

int city_sound_update_die_citizen(void);

int city_sound_update_die_soldier(void);

#endif // CITY_SOUND_H
