#ifndef SCENARIO_TYPES_H
#define SCENARIO_TYPES_H

enum {
    INVASION_TYPE_LOCAL_UPRISING = 1,
    INVASION_TYPE_ENEMY_ARMY = 2,
    INVASION_TYPE_CAESAR = 3,
    INVASION_TYPE_DISTANT_BATTLE = 4,
};

enum {
    EARTHQUAKE_NONE = 0,
    EARTHQUAKE_SMALL = 1,
    EARTHQUAKE_MEDIUM = 2,
    EARTHQUAKE_LARGE = 3
};

#endif // SCENARIO_TYPES_H
