#ifndef WINDOW_EDITOR_MAP_H
#define WINDOW_EDITOR_MAP_H

void window_editor_map_draw_all(void);
void window_editor_map_draw_panels(void);
void window_editor_map_draw(void);

void window_editor_map_show(void);

#endif // WINDOW_EDITOR_MAP_H
