#ifndef FIGURETYPE_MARKET_H
#define FIGURETYPE_MARKET_H

#include "figure/figure.h"

void figure_market_buyer_action(figure *f);

void figure_delivery_boy_action(figure *f);

#endif // FIGURETYPE_MARKET_H
