#ifndef WINDOW_BUILDING_INFO_H
#define WINDOW_BUILDING_INFO_H

void window_building_info_show(int grid_offset);

int window_building_info_get_building_type(void);

void window_building_info_show_storage_orders(void);

#endif // WINDOW_BUILDING_INFO_H
