#ifndef WINDOW_BUILD_MENU_H
#define WINDOW_BUILD_MENU_H

int window_build_menu_image(void);

void window_build_menu_show(int submenu);

void window_build_menu_hide(void);

#endif // WINDOW_BUILD_MENU_H
