#ifndef FIGURE_FORMATION_LAYOUT_H
#define FIGURE_FORMATION_LAYOUT_H

int formation_layout_position_x(int layout, int index);

int formation_layout_position_y(int layout, int index);

#endif // FIGURE_FORMATION_LAYOUT_H
