#ifndef FIGURETYPE_ENEMY_H
#define FIGURETYPE_ENEMY_H

#include "figure/figure.h"

void figure_enemy43_spear_action(figure *f);

void figure_enemy44_sword_action(figure *f);

void figure_enemy45_sword_action(figure *f);

void figure_enemy_camel_action(figure *f);

void figure_enemy_elephant_action(figure *f);

void figure_enemy_chariot_action(figure *f);

void figure_enemy49_fast_sword_action(figure *f);

void figure_enemy50_sword_action(figure *f);

void figure_enemy51_spear_action(figure *f);

void figure_enemy52_mounted_archer_action(figure *f);

void figure_enemy53_axe_action(figure *f);

void figure_enemy_gladiator_action(figure *f);

void figure_enemy_caesar_legionary_action(figure *f);

#endif // FIGURETYPE_ENEMY_H
