#ifndef WIGET_CITY_OVERLAY_HEALTH_H
#define WIGET_CITY_OVERLAY_HEALTH_H

#include "city_overlay.h"

const city_overlay *city_overlay_for_bathhouse(void);

const city_overlay *city_overlay_for_barber(void);

const city_overlay *city_overlay_for_clinic(void);

const city_overlay *city_overlay_for_hospital(void);

#endif // WIGET_CITY_OVERLAY_HEALTH_H
