#ifndef WINDOW_ADVISOR_ENTERTAINMENT_H
#define WINDOW_ADVISOR_ENTERTAINMENT_H

#include "window/advisors.h"

const advisor_window_type *window_advisor_entertainment(void);

#endif // WINDOW_ADVISOR_ENTERTAINMENT_H
