#ifndef WINDOW_BUILDING_GOVERNMENT_H
#define WINDOW_BUILDING_GOVERNMENT_H

#include "common.h"

void window_building_draw_forum(building_info_context *c);

void window_building_draw_senate(building_info_context *c);

void window_building_draw_governor_home(building_info_context *c);

void window_building_draw_garden(building_info_context *c);

void window_building_draw_plaza(building_info_context *c);

void window_building_draw_statue(building_info_context *c);

void window_building_draw_triumphal_arch(building_info_context *c);

#endif // WINDOW_BUILDING_GOVERNMENT_H
