#ifndef WIGET_CITY_OVERLAY_ENTERTAINMENT_H
#define WIGET_CITY_OVERLAY_ENTERTAINMENT_H

#include "city_overlay.h"

const city_overlay *city_overlay_for_entertainment(void);

const city_overlay *city_overlay_for_theater(void);

const city_overlay *city_overlay_for_amphitheater(void);

const city_overlay *city_overlay_for_colosseum(void);

const city_overlay *city_overlay_for_hippodrome(void);

#endif // WIGET_CITY_OVERLAY_ENTERTAINMENT_H
