#ifndef GRAPHICS_IMAGE_GROUP_EDITOR_H
#define GRAPHICS_IMAGE_GROUP_EDITOR_H

enum {
    GROUP_EDITOR_SCENARIO_IMAGE = 22,
    GROUP_EDITOR_BUILDING_CROPS = 23,
    GROUP_EDITOR_EMPIRE_MAP = 47,
    GROUP_EDITOR_SIDE_PANEL = 80,
    GROUP_EDITOR_RESOURCE_ICONS = 129,
    GROUP_EDITOR_EMPIRE_PANELS = 172,
    GROUP_EDITOR_EMPIRE_RESOURCES = 173,
    GROUP_EDITOR_BUILDING_NATIVE = 183,
    GROUP_EDITOR_EMPIRE_FOREIGN_CITY = 223,
    GROUP_EDITOR_TRADE_AMOUNT = 243,
};

#endif // GRAPHICS_IMAGE_GROUP_EDITOR_H
