#ifndef GAME_ORIENTATION_H
#define GAME_ORIENTATION_H

void game_orientation_rotate_left(void);

void game_orientation_rotate_right(void);

void game_orientation_rotate_north(void);

#endif // GAME_ORIENTATION_H
