#ifndef GAME_DIFFICULTY_H
#define GAME_DIFFICULTY_H

int difficulty_starting_favor(void);

int difficulty_sentiment(void);

int difficulty_adjust_money(int money);

int difficulty_adjust_enemies(int enemies);

int difficulty_adjust_wolf_attack(int attack);

#endif // GAME_DIFFICULTY_H
