#ifndef BUILDING_CONSTRUCTION_BUILDING_H
#define BUILDING_CONSTRUCTION_BUILDING_H

#include "building/type.h"

int building_construction_place_building(building_type type, int x, int y);

#endif // BUILDING_CONSTRUCTION_BUILDING_H
