#ifndef WIDGET_CITY_BRIDGE_H
#define WIDGET_CITY_BRIDGE_H

#include "graphics/color.h"

void city_draw_bridge(int x, int y, int grid_offset);

void city_draw_bridge_tile(int x, int y, int bridge_sprite_id, color_t color_mask);

#endif // WIDGET_CITY_BRIDGE_H
