#ifndef SCENARIO_DISTANT_BATTLE_H
#define SCENARIO_DISTANT_BATTLE_H

int scenario_distant_battle_roman_travel_months(void);
int scenario_distant_battle_enemy_travel_months(void);

void scenario_distant_battle_set_roman_travel_months(void);
void scenario_distant_battle_set_enemy_travel_months(void);

void scenario_distant_battle_process(void);

#endif // SCENARIO_DISTANT_BATTLE_H
