#ifndef FIGURETYPE_CARTPUSHER_H
#define FIGURETYPE_CARTPUSHER_H

#include "figure/figure.h"

void figure_cartpusher_action(figure *f);

void figure_warehouseman_action(figure *f);

#endif // FIGURETYPE_CARTPUSHER_H
