#ifndef BUILDING_HOUSE_SERVICE_H
#define BUILDING_HOUSE_SERVICE_H

void house_service_decay_culture(void);

void house_service_decay_tax_collector(void);

void house_service_decay_houses_covered(void);

void house_service_calculate_culture_aggregates(void);

#endif // BUILDING_HOUSE_SERVICE_H
