#ifndef MAP_ROAD_NETWORK_H
#define MAP_ROAD_NETWORK_H

void map_road_network_clear(void);

int map_road_network_get(int grid_offset);

void map_road_network_update(void);

#endif // MAP_ROAD_NETWORK_H
