#ifndef TRANSLATION_COMMON_H
#define TRANSLATION_COMMON_H

#define URL_PATCHES "https://github.com/bvschaik/julius/wiki/Patches"
#define URL_EDITOR "https://github.com/bvschaik/julius/wiki/Editor"

#endif // TRANSLATION_COMMON_H
